package fr.osug.doi;

import gnu.getopt.Getopt;
import gnu.getopt.LongOpt;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author bourgesl
 */
public final class CsvMergePipeline {

    private final static Logger logger = Logger.getLogger(CsvMergePipeline.class.getName());

    public static void main(String[] args) {
        Bootstrap.start();

        // Just leave method if no argument has been given
        if (args == null) {
            showArgumentsHelp();
            System.exit(1);
        }

        // Define default arguments (help, version, log, open file)
        final List<LongOpt> longOpts = new ArrayList<LongOpt>();
        longOpts.clear();
        longOpts.add(new LongOpt("help", LongOpt.NO_ARGUMENT, null, 'h'));
        longOpts.add(new LongOpt("version", LongOpt.NO_ARGUMENT, null, 1));

        // custom arguments
        longOpts.add(new LongOpt("templates", LongOpt.REQUIRED_ARGUMENT, null, 't'));
        longOpts.add(new LongOpt("input", LongOpt.REQUIRED_ARGUMENT, null, 'i'));
        longOpts.add(new LongOpt("output", LongOpt.REQUIRED_ARGUMENT, null, 'o'));
        longOpts.add(new LongOpt("skipMerge", LongOpt.NO_ARGUMENT, null, 's'));
        longOpts.add(new LongOpt("csv", LongOpt.NO_ARGUMENT, null, 'c'));
        longOpts.add(new LongOpt("xml", LongOpt.NO_ARGUMENT, null, 'x'));

        // Instantiate the getopt object
        final LongOpt[] longOptArray = new LongOpt[longOpts.size()];
        longOpts.toArray(longOptArray);

        // Default values:
        String templatePath = null;
        String inputCsvFilePath = null;
        String outputPath = null;
        boolean skipMerge = false;
        boolean saveCSV = false;
        boolean saveXML = false;

        // Parse arguments:
        final Getopt getOpt = new Getopt(Const.VERSION, args, "hv:", longOptArray, true);

        int c; // argument key
        String arg; // argument value

        // While there is a argument key
        while ((c = getOpt.getopt()) != -1) {
            switch (c) {
                // Show the arguments help
                case 'h':
                    showArgumentsHelp();
                    break;

                // Show the name and the version of the program
                case 1:
                    // Show the application name on the shell
                    System.out.println(Const.VERSION);
                    break;

                // Set the logger level
                case 'v':
                    arg = getOpt.getOptarg();

                    if (arg != null) {
                        if (arg.equals("0")) {
                            Bootstrap.setLogLevel(Level.OFF);
                        } else if (arg.equals("1")) {
                            Bootstrap.setLogLevel(Level.SEVERE);
                        } else if (arg.equals("2")) {
                            Bootstrap.setLogLevel(Level.WARNING);
                        } else if (arg.equals("3")) {
                            Bootstrap.setLogLevel(Level.INFO);
                        } else if (arg.equals("4")) {
                            Bootstrap.setLogLevel(Level.FINE);
                        } else if (arg.equals("5")) {
                            Bootstrap.setLogLevel(Level.ALL);
                        } else {
                            showArgumentsHelp();
                        }
                    }
                    break;

                // Other arguments:
                case 't':
                    // -templates <dir>
                    templatePath = getOpt.getOptarg();
                    break;

                case 'i':
                    // -input <csvFile>
                    inputCsvFilePath = getOpt.getOptarg();
                    break;

                case 'o':
                    // -output <dir>
                    outputPath = getOpt.getOptarg();
                    break;

                case 's':
                    // -csv
                    skipMerge = true;
                    break;

                case 'c':
                    // -csv
                    saveCSV = true;
                    break;

                case 'x':
                    // -csv
                    saveXML = true;
                    break;

                default:
                    System.out.println("Unknow command");
                    System.exit(1);
            }
        }

        if (templatePath == null || inputCsvFilePath == null || outputPath == null) {
            logger.severe("Missing required arguments:");
            showArgumentsHelp();
            System.exit(1);
        }
        if (!(saveCSV || saveXML)) {
            logger.warning("Missing output format (-csv or -xml); using xml format.");
            saveXML = true;
        }

        // Main:
        int status = 0;
        try {
            final CsvMergePipeline pipeline = new CsvMergePipeline(
                    templatePath,
                    inputCsvFilePath,
                    outputPath,
                    skipMerge,
                    saveCSV,
                    saveXML);

            pipeline.process();

        } catch (Throwable th) {
            logger.log(Level.SEVERE, "Exception occured:", th);
            status = 1;
        }
        System.exit(status);
    }

    /** Show command arguments help. */
    public static void showArgumentsHelp() {
        System.out.println("---------------------------------- Arguments help ------------------------------");
        System.out.println("| Key          Value           Description                                     |");
        System.out.println("|------------------------------------------------------------------------------|");
        System.out.println("| [-h]                         Show the options help                           |");
        System.out.println("| [-v]         [0|1|2|3|4|5]   Define console logging level                    |");
        System.out.println("| [-version]                   Show application name and version               |");
        System.out.println("|------------------------------------------------------------------------------|");
        System.out.println("| -templates <dir>             Template folder path                            |");
        System.out.println("| -input <file>                CSW GetRecords input (in CSV format)            |");
        System.out.println("| -output <dir>                Output folder path                              |");
        System.out.println("| -output <dir>                Output folder path                              |");
        System.out.println("| [-skipMerge]                 Skip merge templaytes (common, place, id)       |");
        System.out.println("| [-csv]                       CSV Output format                               |");
        System.out.println("| [-xml]                       XML Output format                               |");
        System.out.println("|------------------------------------------------------------------------------|");
        System.out.println("| LOG LEVELS : 0 = OFF, 1 = SEVERE, 2 = WARNING, 3 = INFO, 4 = FINE, 5 = ALL   | \n");
        System.out.println("|------------------------------------------------------------------------------|");

        // Exit the application
        System.exit(0);
    }

    // members
    private final File inputCsvFile;
    private final DoiTemplates templates;
    private final File output;
    private final boolean doMerge;
    private final boolean saveCSV;
    private final boolean saveXML;

    public CsvMergePipeline(final String templatePath,
                            final String inputCsvFilePath,
                            final String outputPath,
                            final boolean skipMerge,
                            final boolean saveCSV,
                            final boolean saveXML) throws IOException {

        this.inputCsvFile = DoiCsvUtil.getFile(inputCsvFilePath);

        this.templates = (skipMerge) ? null : DoiTemplates.getInstance(templatePath);

        this.output = new File(outputPath);
        if (!output.exists()) {
            output.mkdirs();
        }

        this.doMerge = !skipMerge;
        this.saveCSV = saveCSV;
        this.saveXML = saveXML;
    }

    public void process() throws IOException {
        /*
        Spliter et produire 1 fichier XML dataCite par jeu.
        Ex: AMMA-CATCH.CL.Run_O.xml
         */
        final List<DoiCsvData> csvDatas = DoiCsvSplitter.split(inputCsvFile);

        for (DoiCsvData data : csvDatas) {
            if (logger.isLoggable(Level.FINE)) {
                logger.log(Level.FINE, "process: data:\n{0}", data);
            }
            if (!data.isEmpty()) {
                final String id = data.getId();
                if (id == null) {
                    // Missing identifier
                    logger.log(Level.SEVERE, "Missing ''"
                            + Const.KEY_IDENTIFIER + "'' in dataset:\n{0}", data);
                    // skip
                    continue;
                }
                logger.log(Level.INFO, "process: dataset[{0}].", id);

                if (doMerge) {
                    /*                
                    ### 2/ Fusionner avec template observatoire : template_obs.csv

                    Informations disjointes
                     */
                    DoiCsvData t = templates.getBase();
                    if (t == null) {
                        logger.log(Level.SEVERE, "Missing base template");
                        // go on ...
                    } else {
                        data.mergeFrom(t);
                    }

                    /*
                    ### 3/ Traitement spécifiques: fusionner avec template pays

                    Fusionner avec template pays (template_benin.csv):

                    Trouver le template du pays qui contient geoLocationPlace;Benin
                     */
                    final String geoLocationPlace = data.getFirstGeoLocationPlace();
                    if (geoLocationPlace == null) {
                        // Missing country
                        logger.log(Level.SEVERE, "Missing ''"
                                + Const.KEY_GEO_LOCATION_PLACE + "'' in dataset:\n{0}", data);
                        // go on ...
                    } else {
                        t = templates.getByGeoLocationPlace(geoLocationPlace);
                        if (t == null) {
                            logger.log(Level.SEVERE, "Missing country template for ''{0}'' ...", geoLocationPlace);
                            // go on ...
                        } else {
                            data.mergeFrom(t);
                        }
                    }

                    /*
                    ### 4/ Traitement spécifiques: fusionner avec template jeu

                    Trouver le nom du jeu dans le XML:
                    <identifier identifierType="DOI">10.5072/AMMA-CATCH.CL.Run_O</identifier>

                    Trouver le template du jeu qui contient identifier:DOI;10.5072/AMMA-CATCH.CL.Run_O
                     */
                    t = templates.getByIdentifier(id);
                    if (t == null) {
                        logger.log(Level.WARNING, "Missing dataset template for id ''{0}'' ...", id);
                        // go on ...
                    } else {
                        logger.log(Level.INFO, "Using dataset template for id ''{0}'' ...", id);
                        data.mergeFrom(t);
                    }
                }

                // Save metadata for dataset:
                saveData(data);
            }
        }
    }

    private void saveData(final DoiCsvData data) throws IOException {
        // Sort keys:
        data.sort();

        // File base name:
        final String baseName = removeDOIPrefix(data.getId());

        if (saveCSV) {
            // Save file as CSV:
            final File fileCSV = new File(output, baseName + ".csv");
            logger.log(Level.INFO, "Writing: {0}", fileCSV.getAbsolutePath());
            DoiCsvUtil.write(data, fileCSV);
        }

        if (saveXML) {
            // Save file as XML:
            final File fileXML = new File(output, baseName + ".xml");
            logger.log(Level.INFO, "Writing: {0}", fileXML.getAbsolutePath());
            DoiCsvToXml.write(data, fileXML);
        }
    }

    private static String removeDOIPrefix(final String identifier) {
        final int pos = identifier.indexOf('/');

        if (pos != -1) {
            return identifier.substring(pos + 1);
        }
        return identifier;
    }
}
