package fr.osug.doi;

import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author bourgesl
 */
public final class DoiCsvData {

    private final static Logger logger = Logger.getLogger(DoiCsvData.class.getName());

    private final String filePath;
    private final List<String[]> rows;

    DoiCsvData(final List<String[]> rows) {
        this(null, rows);
    }

    DoiCsvData(final String filePath, final List<String[]> rows) {
        this.filePath = filePath;
        this.rows = rows;
    }

    public String getFilePath() {
        return filePath;
    }

    public boolean isEmpty() {
        return rows == null || rows.isEmpty();
    }

    public List<String[]> getRows() {
        return rows;
    }

    @Override
    public String toString() {
        if (isEmpty()) {
            return "empty";
        }
        final StringBuilder sb = new StringBuilder(1024);

        for (int i = 0, size = rows.size(); i < size; i++) {
            final String[] cols = rows.get(i);
            if (cols != null) {
                final int len = cols.length;
                if (len == 2) {
                    sb.append('[').append(cols[0]).append("]=").append(cols[1]).append('\n');
                } else {
                    sb.append('(').append(len).append(") {");
                    for (int j = 0; j < len; j++) {
                        if (j != 0) {
                            sb.append('|');
                        }
                        sb.append(cols[j]);
                    }
                    sb.append("}\n");
                }
            }
        }
        return sb.toString();
    }

    public String getValue(final String key) {
        for (String[] cols : rows) {
            if (cols != null && cols.length >= 2 && key.equalsIgnoreCase(cols[0])) {
                // first key found, return value:
                return cols[1];
            }
        }
        return null;
    }

    public String getId() {
        return getValue(Const.KEY_IDENTIFIER);
    }

    public String getFirstGeoLocationPlace() {
        return getValue(Const.KEY_GEO_LOCATION_PLACE);
    }

    public void trim() {
        for (ListIterator<String[]> it = rows.listIterator(); it.hasNext();) {
            final String[] cols = it.next();
            if (cols == null || cols.length == 0) {
                // remove line:
                it.remove();
                continue;
            }
            // Find first empty value:
            int last = -1;
            for (int i = 0; i < cols.length; i++) {
                String v = cols[i].trim();
                if (v.isEmpty() || v.charAt(0) == Const.COMMENT) {
                    break;
                }
                last = i;
            }

            int n = last + 1;
            if (n < 2) {
                // first pair is invalid:
                if (logger.isLoggable(Level.FINE)) {
                    logger.fine("trim: invalid first pair:" + Arrays.toString(cols));
                }
                // remove line:
                it.remove();
                continue;
            }

            if (n % 2 != 0) {
                // ignore last column (comment):
                if (logger.isLoggable(Level.FINE)) {
                    logger.fine("trim: even columns:" + Arrays.toString(cols));
                }
                n -= 1;
            }

            if (n != cols.length) {
                // resize array:
                final String[] newCols = Arrays.copyOf(cols, n);
                if (logger.isLoggable(Level.FINE)) {
                    logger.fine("trim: resize columns:" + Arrays.toString(cols) + " to: " + Arrays.toString(newCols));
                }
                it.set(newCols);
            }
        }
    }

    public void mergeFrom(final DoiCsvData src) {
        for (String[] srcCols : src.getRows()) {
            if (srcCols != null) {
                // look for matching in this CSV:
                boolean match = false;
                for (String[] toCols : rows) {
                    if (Arrays.equals(srcCols, toCols)) {
                        match = true;
                        break;
                    }
                }

                if (!match) {
                    // avoid duplicates
                    rows.add(srcCols);
                }
            }
        }
    }

    public void sort() {
        Collections.sort(rows, RowComparator.singleton);
    }

    final static class RowComparator implements Comparator<String[]> {

        private final static Map<String, Integer> orderIndex = new HashMap<String, Integer>(128);

        final static RowComparator singleton = new RowComparator();

        static {
            int i = 1;
            for (String key : Const.KEY_ORDER) {
                orderIndex.put(key, i++);
            }
        }

        @Override
        public int compare(final String[] row1, final String[] row2) {
            int idx1 = -1;
            int idx2 = -1;
            if (row1 != null && row1.length != 0) {
                idx1 = getIndex(row1[0]);
            }
            if (row2 != null && row2.length != 0) {
                idx2 = getIndex(row2[0]);
            }
            return Integer.compare(idx1, idx2);
        }

        private static int getIndex(final String key) {
            final Integer idx = orderIndex.get(key);
            if (idx == null) {
                logger.log(Level.INFO, "Missing key [{0}] !", key);
                return -1;
            }
            return idx.intValue();
        }
    }
}
