/*******************************************************************************
 * JMMC project ( http://www.jmmc.fr ) - Copyright (C) CNRS.
 ******************************************************************************/
package fr.osug.doi;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author bourgesl
 */
public class DoiTemplates {

    private static Logger logger;

    private static volatile DoiTemplates singleton = null;

    public synchronized static DoiTemplates getInstance(final String templatePath) throws IOException {
        if (singleton == null) {
            logger = Logger.getLogger(DoiTemplates.class.getName());
            singleton = new DoiTemplates(templatePath);
        }
        return singleton;
    }

    /* members */
    private DoiCsvData base = null;
    private final Map<String, DoiCsvData> geoLocationPlaces = new HashMap<String, DoiCsvData>();
    private final Map<String, DoiCsvData> identifiers = new HashMap<String, DoiCsvData>();

    private DoiTemplates(final String templatePath) throws IOException {
        initialize(templatePath);
    }

    private void initialize(final String templatePath) throws IOException {
        final File templateDir = DoiCsvUtil.getDirectory(templatePath);

        logger.log(Level.INFO, "Loading templates from: {0}", templateDir.getAbsolutePath());

        final File[] csvFiles = getSortedCSVFiles(templateDir);

        for (File file : csvFiles) {
            final DoiCsvData data = DoiCsvUtil.read(file);
            data.trim();

            if (!data.isEmpty()) {
                if (logger.isLoggable(Level.FINE)) {
                    logger.log(Level.FINE, "initialize: [{0}] \n{1}",
                            new Object[]{data.getFilePath(), data});
                }

                final String id = data.getId();
                if (id != null) {
                    identifiers.put(id, data);
                    continue;
                }

                final String geoLocationPlace = data.getFirstGeoLocationPlace();
                if (geoLocationPlace != null) {
                    geoLocationPlaces.put(geoLocationPlace, data);
                    continue;
                }

                // Anything left:
                if (base == null) {
                    base = data;
                } else {
                    logger.severe("initialize: several files without '" + Const.KEY_IDENTIFIER
                            + "' or '" + Const.KEY_GEO_LOCATION_PLACE + "' ie several base templates:\n"
                            + base.getFilePath() + "\n" + data.getFilePath());
                }
            }
        }

        if (logger.isLoggable(Level.FINE)) {
            logger.log(Level.FINE, "base: \n{0}", base);
            logger.log(Level.FINE, "identifiers: \n{0}", identifiers);
            logger.log(Level.FINE, "geoLocationPlaces: \n{0}", geoLocationPlaces);
        }
    }

    public DoiCsvData getBase() {
        return base;
    }

    public DoiCsvData getByIdentifier(final String id) {
        return identifiers.get(id);
    }

    public DoiCsvData getByGeoLocationPlace(final String geoLocationPlace) {
        return geoLocationPlaces.get(geoLocationPlace);
    }

    private static File[] getSortedCSVFiles(final File directory) {
        final File[] dataFiles = directory.listFiles(new FilenameFilter() {
            @Override
            public boolean accept(File dir, String name) {
                return name.matches(".*\\.csv");
            }
        });

        // Sort file names:
        Arrays.sort(dataFiles);

        return dataFiles;
    }

}
